/*
 * pdsvector.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsvector.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con vectores.
 *  
 *  <br>Estas funciones trabajan con un vector de la forma.<br>
 *  \image html vector.png "Vector de Nel elementos"
 *  \b Nel es el número de elementos.
 */

#ifndef __PDSVECTOR_H__
#define __PDSVECTOR_H__

#ifdef __cplusplus
extern "C" {
#endif 

#include <stdio.h>
#include <stdlib.h>
#include <pds/pdsraglobal.h>

#ifndef TRUE
	#define TRUE 1
#endif

#ifndef FALSE
	#define FALSE 0
#endif

/** \defgroup PdsVectorGroup Módulo PdsVector.
 *  \brief Funciones que trabajan con vectores.
 *  
 *  <br>Estas funciones trabajan con un vector de la forma.<br>
 *  <center>
 *  \image html vector.png "Vector de \b Nel elementos"
 *  </center>
 * @{
 */

/*! \def _VECSETVAL(Vector,id,Val)
 *  Es equivalente a Vector{id}=Val.
 *  \ingroup PdsVectorGroup
*/
#define _VECSETVAL(Vector,id,Val) pds_vector_set_value(Vector,id,Val)

/*! \struct PdsVector
 *  \brief Estructura de tipo  PdsVector .
 *
 *  Esta estructura genera una matriz de \b Nel elementos.
 *  Para usar incluir la cabecera pds/pdsvector.h.
 *  \ingroup PdsVectorGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Un arreglo unidimencional de \b Nel elementos. */
	PdsRaReal *V;
	/*! Número de elementos. */
	PdsRaNatural Nel;
}PdsVector;

/** \fn PdsVector *pds_vector_new(PdsRaNatural Nel)
 *  \brief Crea un vector de tipo PdsVector e inicia con cero todos los elementos.
 *  \param[in] Nel Es el número de elementos del vector.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new(PdsRaNatural Nel);


/** \fn PdsVector *pds_vector_new_array1d(const PdsRaReal *Array1D,PdsRaNatural N)
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con el contenido
 *  del arreglo de valores reales Array1D.
 *  
 *  exemplo:
\code{.c}
    PdsVector *V=NULL;
    PdsRaReal A[]={-1,1.2,-3.0};
    V=pds_vector_new_array1d(A,3);
\endcode
 *  \warning { Debes tener mucho cuidado con esta función, entregando el número de elementos adecuado. }
 *  \param[in] Array1D Es el arreglo unidimensional que se usará como fuente.
 *  \param[in] N Es el número de elementos del arreglo unidimensional que se usará como fuente.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_array1d(const PdsRaReal *Array1D,PdsRaNatural N);


/** \fn PdsVector *pds_vector_new_args(PdsRaNatural N,...)
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con los valores
 *  del segundo parametro de entrada en adelante.
 *  
 *  exemplo:
\code{.c}
    PdsVector *V=NULL;
    V=pds_vector_new_args(3,-1,1.2,-3.0);
\endcode
 *  \warning { Debes tener mucho cuidado con esta función, entregando el número de elementos adecuado. }
 *  \param[in] N Es el número de elementos del vector y el numero de parametros
 *  de entrada adicionales.
 *  \param[in] ... Parametros adicionales de tipo double.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_args(PdsRaNatural N,...);


/** \fn PdsVector *pds_vector_new_vector(const PdsVector *VecSrc)
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con el contenido
 *  de el vector VecSrc.
 *  \param[in] VecSrc Es el vector que se usará como fuente.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_vector(const PdsVector *VecSrc);


/** \fn PdsVector *pds_vector_new_load_data(const char* datafile)
 *  \brief Crea un nuevo vector e inicia los datos con los elementos del archivo
 *  \b datafile.
 *  Usa TAB o un salto de linea como delimitador de elemento.
 *  \param[in] datafile Nombre del archivo de donde se cargará los datos iniciales
 *  del vector.
 *  \return Un puntero que apunta a la dirección del nuevo vector, si todo fue bien, 
 *  o NULL si no. (ej. datafile==NULL)
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_load_data(const char* datafile);


/** \fn PdsVector *pds_vector_new_load_data_line(const char* datafile,PdsRaNatural line)
 *  \brief Crea un nuevo vector e inicia los datos con los elementos de la linea
 *  \b line del archivo \b datafile.
 *  Usa TAB como delimitador de elemento y un salto de linea como delimitador de linea.
 *  La primera linea es \b line=0 .
 *  Si se pide una linea inexistente la funcion retorna NULL.
 *  \param[in] datafile Nombre del archivo de donde se cargará los datos iniciales
 *  del vector.
 *  \param[in] line Número de linea que se cargará, line inicia en cero. 
 *  \return Un puntero que apunta a la dirección del nuevo vector, si todo fue bien, 
 *  o NULL si no. (ej. datafile==NULL o line inexistente)
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_load_data_line(const char* datafile,PdsRaNatural line);


/** \fn PdsVector *pds_vector_new_load_data_col(const char* datafile,PdsRaNatural col)
 *  \brief Crea un nuevo vector e inicia los datos con los elementos de la columna
 *  \b col del archivo \b datafile.
 *  Usa TAB como delimitador de columna y un salto de linea como delimitador de linea.
 *  La primera columna es \b col=0 .
 *  Si se pide una linea inexistente la función retorna NULL
 *  \param[in] datafile Nombre del archivo de donde se cargará los datos iniciales
 *  del vector.
 *  \param[in] col Número de columna que se cargará, col inicia en cero. 
 *  \return Un puntero que apunta a la dirección del nuevo vector, si todo fue bien, 
 *  o NULL si no. (ej. datafile==NULL o col inexistente)
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_load_data_col(const char* datafile,PdsRaNatural col);


/** \fn int pds_vector_init_value(PdsVector *Vector,PdsRaReal Val)
 *  \brief Inicia el vector \b Vector de tipo PdsVector *. <br><b>Vector{:}=Val</b>.
 *  \param[in,out] Vector El vector a inicializar.
 *  \param[in] Val Es el valor de inicialización de los elementos.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_init_value(PdsVector *Vector,PdsRaReal Val);


/** \fn int pds_vector_init_vector(PdsVector *Vector,const PdsVector *VecSrc)
 *  \brief Se copia el contenido del vector \b VecSrc a el vector \b Vector.
 *  Si los tamaños son diferentes se intersectan los tamaños y se hace la copia en la
 *  intersección solamente. <br><b>Vector=VecSrc</b>.
 *  \param[in,out] Vector Es el vector a iniciar.
 *  \param[in] VecSrc Es el vector fuente de la copia.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_init_vector(PdsVector *Vector,const PdsVector *VecSrc);


/** \fn int pds_vector_add_vector(PdsVector *Vector,const PdsVector *VecSrc)
 *  \brief Suma al vector \b Vector el vector \b VecSrc, el resultado es copiado a \b Vector.
 *  Si los tamaños son diferentes intersecta los tamaños y hace la copia en la
 *  intersección solamente. <br><b>Vector=Vector+VecSrc</b>.
 *  \param[in,out] Vector Es el vector de destino y operando.
 *  \param[in] VecSrc Es el Vector fuente del operando.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_add_vector(PdsVector *Vector,const PdsVector *VecSrc);


/** \fn int pds_vector_add_factor_vector(PdsVector *Vector,PdsRaReal Alpha,const PdsVector *VecSrc)
 *  \brief Suma al vector \b Vector el vector \b Alpha*VecSrc, el resultado es copiado a \b Vector.
 *  Si los tamaños son diferentes se intersectan los tamaños y se
 *  hace la copia en la intersección solamente. <br><b>Vector=Vector+Alpha*VecSrc</b>.
 *  \param[in,out] Vector Es el vector de destino y operando.
 *  \param[in] Alpha Factor que multiplica al operando.
 *  \param[in] VecSrc Es el Vector fuente del operando.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_add_factor_vector(PdsVector *Vector,PdsRaReal Alpha,const PdsVector *VecSrc);


/** \fn int pds_vector_add_value(PdsVector *Vector,PdsRaReal Val)
 *  \brief Suma al vector \b Vector el valor \b Val y lo carga en \b Vector. <br><b>Vector{:}=Vector{:}+Val</b>.
 *  \param[in,out] Vector Es el vector a sumar.
 *  \param[in] Val Es el valor a acumular.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_add_value(PdsVector *Vector,PdsRaReal Val);


/** \fn int pds_vector_sub_vector(PdsVector *Vector,const PdsVector *VecSrc)
 *  \brief Resta al vector de tipo puntero PdsVector con otro vector y lo copia en Vector.
 *  Si los tamaños son diferentes intersecta los tamaños y hace la copia en la
 *  intersección solamente. <br><b>Vector=Vector-VecSrc</b>.
 *  \param[in,out] Vector Es el vector de destino y operando.
 *  \param[in] VecSrc Es el Vector fuente del operando.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_sub_vector(PdsVector *Vector,const PdsVector *VecSrc);


/** \fn int pds_vector_mul_vector_elements(PdsVector *Vector,const PdsVector *VecSrc)
 *  \brief Multiplica elemento a elemento el vector \b Vector con el vector 
 *  \b VecSrc, el resultado es cargado a \b Vector.
 *  Si los tamaños son diferentes intersecta los tamaños y hace la copia en la
 *  intersección solamente. <br><b>Vector=Vector.VecSrc</b>.
 *  \param[in,out] Vector Es el vector de destino y operando.
 *  \param[in] VecSrc Es el Vector fuente del operando.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_mul_vector_elements(PdsVector *Vector,const PdsVector *VecSrc);


/** \fn int pds_vector_mul_value(PdsVector *Vector,PdsRaReal Val)
 *  \brief Multiplica al vector \b Vector por un escalar. <br><b>Vector=Val*Vector</b>.
 *  \param[in,out] Vector Es el vector a multiplicar.
 *  \param[in] Val Es el valor que multiplicará a los elementos del vector.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_mul_value(PdsVector *Vector,PdsRaReal Val);


/** \fn int pds_vector_div_value(PdsVector *Vector,PdsRaReal Val)
 *  \brief Divide al vector \b Vector por un escalar. Carga el resultado en el
 *  propio vector. <br><b>Vector=Vector/Val</b>.
 *  \param[in,out] Vector Es el vector a dividir.
 *  \param[in] Val Es el valor que dividirá a los elementos del vector.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_div_value(PdsVector *Vector,PdsRaReal Val);


/** \fn int pds_vector_dot_vector(const PdsVector *Vector1,const PdsVector *Vector2,PdsRaReal *Val)
 *  \brief Ejecuta el producto punto entre el \b Vector1 y el \b Vector2.
 *  El resultado es cargado en \b Val.
 *  Los tamaños deben ser iguales. <br><b>Val=(Vector1,Vector2) </b>.
 *  \param[in] Vector1 Es el vector operando.
 *  \param[in] Vector2 Es el Vector operando.
 *  \param[out] Val El producto punto entre \b Vector1 y \b Vector2.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_dot_vector(const PdsVector *Vector1,const PdsVector *Vector2,PdsRaReal *Val);


/** \fn int pds_vector_printf(const PdsVector *Vector)
 *  \brief Imprime en pantalla un vector de tipo puntero PdsVector.
 *  \param[in] Vector El vector a imprimir en pantalla.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_printf(const PdsVector *Vector);


/** \fn int pds_vector_fprintf(const PdsVector *Vector,FILE *fd)
 *  \brief Imprime en el archivo al que apunta \b fd el contenido de un vector \b Vector.
 *  \b fd es un manejador de archivo a un archivo de texto.
 *  Para usar:
\code{.h}
FILE *fd=NULL;
fd=fopen ("file.txt", "w+");
pds_vector_fprintf(Vector,fd);
fclose(fd);
\endcode
 *  \param[in] Vector El vector a imprimir en el archivo de texto.
 *  \param[in,out] fd Un manejador de archivo de texto.
 *  \return TRUE si todo fue bien o FALSE si no.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_fprintf(const PdsVector *Vector,FILE *fd);


/** \fn int pds_vector_fscanf(PdsVector *Vector, FILE *fd)
 *  \brief Inicializa un vector \b Vector con los datos del archivo de texto apuntado por \b fd.
 *  Si la cantidad de elementos no es suficiente, entonces el vector es completado con ceros.
 *  Usa TAB o un salto de linea como delimitador de elemento.
 *  Para usar:
\code{.h}
FILE *fd=NULL;
fd=fopen ("file.txt", "r");
pds_vector_fscanf(Vector,fd);
fclose(fd);
\endcode
 *  \param[in] Vector Vector en donde se cargarán los datos.
 *  \param[in] fd Apuntador del archivo de donde se cargarán los datos iniciales
 *  del vector.
 *  \return TRUE si todo fue bien o FALSE si no. (ej. Vector==NULL o fd==NULL)
 *  \ingroup PdsVectorGroup
 */
int pds_vector_fscanf(PdsVector *Vector, FILE *fd);


/** \fn int pds_vector_fwrite(PdsVector *Vector, FILE *fd)
 *  \brief Escribe los datos de un vector en el archivo binario apuntado por \b fd.
 *  Para usar:
\code{.h}
FILE *fd=NULL;
fd=fopen ("file.dat", "wb");
pds_vector_fwrite(Vector, fd);
fclose(fd);
\endcode
 *  \param[in] Vector Vector de donde se leerán los datos.
 *  \param[in] fd Apuntador del archivo binario de donde se escribirán los datos.
 *  \return TRUE si todo fue bien o FALSE si no. (ej. Vector==NULL o fd==NULL)
 *  \ingroup PdsVectorGroup
 */
int pds_vector_fwrite(PdsVector *Vector, FILE *fd);


/** \fn int pds_vector_fread(PdsVector *Vector, FILE *fd)
 *  \brief Inicializa un vector \b Vector con los datos del archivo binario apuntado por \b fd.
 *  Si la cantidad de datos en el archivo es menor a la cantidad de elementos 
 *  del vector \b Vector, entonces se completa con ceros.
 *  Para usar:
\code{.h}
FILE *fd=NULL;
fd=fopen ("file.dat", "rb");
pds_vector_fread(Vector, fd);
fclose(fd);
\endcode
 *  \param[in] Vector Vector en donde se cargaran los datos.
 *  \param[in] fd Apuntador del archivo binario de donde se cargarán los datos 
 *  iniciales del vector.
 *  \return TRUE si todo fue bien o FALSE si no. (ej. Vector==NULL o fd==NULL)
 *  \ingroup PdsVectorGroup
 */
int pds_vector_fread(PdsVector *Vector, FILE *fd);


/** \fn int pds_vector_norm_vector(const PdsVector *Vector, PdsRaReal *m)
 *  \brief Devuelve la norma del vector Vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] m El valor de la norma del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_norm_vector(const PdsVector *Vector, PdsRaReal *m);


/** \fn int pds_vector_norm2_vector(const PdsVector *Vector, PdsRaReal *m)
 *  \brief Devuelve la norma^2 del vector Vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] m El valor de la norma^2 del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_norm2_vector(const PdsVector *Vector, PdsRaReal *m);


/** \fn int pds_vector_rms_vector(const PdsVector *Vector, PdsRaReal *m)
 *  \brief Devuelve el valor raíz cuadrático medio del vector Vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] m El valor rms del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_rms_vector(const PdsVector *Vector, PdsRaReal *m);


/** \fn int pds_vector_rms_vectors(const PdsVector *VectorA, const PdsVector *VectorB, PdsRaReal *m)
 *  \brief Devuelve el valor raíz cuadrático medio de la diferencia de los vectores.
 *  \param[in] VectorA El vector en consulta.
 *  \param[in] VectorB El vector en consulta.
 *  \param[out] m El valor rms de la diferencias de vectores.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL, o diferentes tamaños). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_rms_vectors(const PdsVector *VectorA, const PdsVector *VectorB, PdsRaReal *m);


/** \fn int pds_vector_rms_factor_vectors(const PdsVector *VectorA, const PdsVector *VectorB, PdsRaReal *m)
 *  \brief Devuelve el valor raíz cuadrático medio de la diferencia de los vectores
 *  entre el valor raíz cuadrático medio de VectorB, si este valor es cero, se 
 *  usa entonces el valor raíz cuadrático medio de VectorA.
 *
 *  \f[ 
	\frac{\sqrt{\frac{\sum_{i=0}^{Nel-1}{(VectorA[i]-VectorB[i])^2}}{Nel}}}
	{\sqrt{\frac{\sum_{i=0}^{Nel-1}{VectorB[i]^2}}{Nel}}} 
    \f]
 *  \param[in] VectorA El vector en consulta.
 *  \param[in] VectorB El vector en consulta.
 *  \param[out] m El valor factor rms de la diferencias de vectores.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL, o diferentes tamaños). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_rms_factor_vectors(const PdsVector *VectorA, const PdsVector *VectorB, PdsRaReal *m);

/** \fn int pds_vector_mean_vector(const PdsVector *Vector, PdsRaReal *m)
 *  \brief Devuelve el valor medio del vector (X) Vector.
 *
 *  \f[ \eta=\frac{\sum_{i=0}^{Nel-1} {X_i}}{Nel} \f]
 *  \param[in] Vector El vector en consulta.
 *  \param[out] m El valor medio del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_mean_vector(const PdsVector *Vector, PdsRaReal *m);


/** \fn int pds_vector_var_vector(const PdsVector *Vector, PdsRaReal *v)
 *  \brief Devuelve el valor de la varianza muestral \f$\sigma^2\f$ del vector Vector (X).
 *
 *  \f[ \eta=\frac{\sum_{i=0}^{Nel-1} {X_i}}{Nel} \f]
 *  \f[ \sigma^2=\frac{\sum_{i=0}^{Nel-1} {(X_i-\eta)^2}}{Nel-1} \f]
 *  \param[in] Vector El vector en consulta.
 *  \param[out] v El valor de la varianza del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_var_vector(const PdsVector *Vector, PdsRaReal *v);


/** \fn int pds_vector_var_vector(const PdsVector *Vector, PdsRaReal *v)
 *  \brief Devuelve el valor del desvio padron \f$\sigma\f$ del vector Vector (X).
 *
 *  \f[ \eta=\frac{\sum_{i=0}^{Nel-1} {X_i}}{Nel} \f]
 *  \f[ \sigma^2=\frac{\sum_{i=0}^{Nel-1} {(X_i-\eta)^2}}{Nel} \f]
 *  \param[in] Vector El vector en consulta.
 *  \param[out] v El valor de desvio padron del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_std_vector(const PdsVector *Vector, PdsRaReal *v);


/** \fn int pds_vector_cov_vector (const PdsVector *VectorX,const PdsVector *VectorY, PdsRaReal *c)
 *  \brief Devuelve el valor de la covarianza muestral de los vectores \b VectorX y \b VectorY.
 *
 *  \f[ cov(X,Y)=\frac{\sum_{i=0}^{Nel-1} {(X_i -{\eta}_x)(Y_i -{\eta}_y)}}{Nel-1} \f]
 *
 *  Siendo \f$\eta_x\f$ y \f$\eta_y\f$:
 *  \f[ \eta_x=\frac{\sum_{i=0}^{Nel-1} {X_i}}{Nel} \f]
 *  \f[ \eta_y=\frac{\sum_{i=0}^{Nel-1} {Y_i}}{Nel} \f]
 *
 *  \param[in] VectorX El vector en consulta.
 *  \param[in] VectorY El vector en consulta.
 *  \param[out] c El valor de la covarianza de los vectores \b VectorX y \b VectorY.
 *  \return TRUE si todo fue bien o FALSE si no (ej: VectorX==NULL, VectorY==NULL o
 *  longitudes distintas). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_cov_vector (const PdsVector *VectorX,const PdsVector *VectorY, PdsRaReal *c);


/** \fn int pds_vector_cor_vector (const PdsVector *VectorX,const PdsVector *VectorY, PdsRaReal *c)
 *  \brief Devuelve el valor del coeficiente de correlación de los vectores \b VectorX y \b VectorY.
 *
 *  \f[ cor(X,Y)=\frac{\sum_{i=0}^{Nel-1} {(X_i -{\eta}_x)(Y_i -{\eta}_y)}}{\sqrt{\sum_{i=0}^{Nel-1} {(X_i -{\eta}_x)^2}}\sqrt{\sum_{i=0}^{Nel-1} {(Y_i -{\eta}_y)^2}}} \f]
 *
 *  Siendo \f$\eta_x\f$ y \f$\eta_y\f$:
 *  \f[ \eta_x=\frac{\sum_{i=0}^{Nel-1} {X_i}}{Nel} \f]
 *  \f[ \eta_y=\frac{\sum_{i=0}^{Nel-1} {Y_i}}{Nel} \f]
 *
 *  \param[in] VectorX El vector en consulta.
 *  \param[in] VectorY El vector en consulta.
 *  \param[out] c El valor de la correlación de los vectores \b VectorX y \b VectorY.
 *  \return TRUE si todo fue bien o FALSE si no (ej: VectorX==NULL, VectorY==NULL o
 *  longitudes distintas). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_cor_vector (const PdsVector *VectorX,const PdsVector *VectorY, PdsRaReal *c);


/** \fn PdsVector* pds_vector_new_conv(const PdsVector *Vector1,const PdsVector *Vector2)
 *  \brief Devuelve un nuevo vector con la convolución de Vector1 y Vector2 de
 *  longitudes N1 y N2 respectivamente.
 *
 *  \f[ C(m)=\sum_{i=0, \forall 0 \leq (m-i) \leq N2-1}^{N1-1} Vector1(i)*Vector2(m-i) \f]
 *
 *  \param[in] Vector1 Un vector a convolucionar.
 *  \param[in] Vector2 Un vector a convolucionar.
 *  \return Retorna un vector con la convolución de Vector1 e Vector2 o NULL en 
 *  caso de error. 
 *  \ingroup PdsVectorGroup
 */
PdsVector* pds_vector_new_conv(const PdsVector *Vector1,const PdsVector *Vector2);


/** \fn int pds_vector_get_value(const PdsVector *Vector, PdsRaNatural x, PdsRaReal *m)
 *  \brief Devuelve el valor en la posición (x) del vector Vector.
 *  (x) inicia con (0).
 *  \param[in] Vector El vector en consulta.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[out] m El valor en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces carga 0 en m, en caso de error de vector nulo carga cero en m.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_get_value(const PdsVector *Vector, PdsRaNatural x, PdsRaReal *m);


/** \fn int pds_vector_set_value(PdsVector *Vector, PdsRaNatural x, PdsRaReal m)
 *  \brief Escribe el valor m en la posición (x) del vector Vector.
 *  (x) inicia con (0).
 *  \param[in,out] Vector El vector a escribir.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[in] m El valor en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces no hace nada y no se considera como error.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_set_value(PdsVector *Vector, PdsRaNatural x, PdsRaReal m);


/** \fn int pds_vector_get_nelements(const PdsVector *Vector,PdsRaReal *Nelements)
 *  \brief Devuelve el número de elementos del vector Vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] Nelements En donde se guardará el número de elementos del vector Vector.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_get_nelements(const PdsVector *Vector,PdsRaReal *Nelements);

/** \fn int pds_vector_get_min_value(const PdsVector *Vector, PdsRaReal *m)
 *  \brief Devuelve el valor mínimo de todos los elementos del vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] m El valor mínimo de todos los elementos del vector Vector. En 
 *  caso de error por vector nulo carga cero en m.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_get_min_value(const PdsVector *Vector, PdsRaReal *m);


/** \fn int pds_vector_get_max_value(const PdsVector *Vector, PdsRaReal *m)
 *  \brief Devuelve el valor máximo de todos los elementos del vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] m El valor máximo de todos los elementos del vector Vector. En 
 *  caso de error por vector nulo carga cero en m.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_get_max_value(const PdsVector *Vector, PdsRaReal *m);


/** \fn int pds_vector_swap_elements(PdsVector *Vector, PdsRaNatural X, PdsRaNatural Y)
 *  \brief Intercambia los elementos de la posición X e Y del vector.
 *  \param[in] Vector El vector en consulta.
 *  \param[out] X Posicion del vector. X in [0,Nel-1].
 *  \param[out] Y Posicion del vector. Y in [0,Nel-1].
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL o X,Y fuera de rango). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_swap_elements(PdsVector *Vector, PdsRaNatural X, PdsRaNatural Y);


/** \fn int pds_vector_asc_order(PdsVector *Vector)
 *  \brief Ordena ascendentemente todos los elementos del vector.
 *  \param[in,out] Vector El vector en consulta.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_asc_order(PdsVector *Vector);


/** \fn int pds_vector_desc_order(PdsVector *Vector)
 *  \brief Ordena descendentemente todos los elementos del vector.
 *  \param[in,out] Vector El vector en consulta.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_desc_order(PdsVector *Vector);


/** \fn int pds_vector_set_bcd(PdsVector *Vector,PdsRaNatural n,PdsRaNatural b0,PdsRaNatural b1)
 *  \brief Escribe en formato BCD el numero n en el vector Vector.
 *  \param[in,out] Vector El vector a escribir.
 *  \param[in] n EL número decimal e escribir en BCD en el vector Vector
 *  \param[in] b0 El valor que será escrito cuando se representa a un bit 0.
 *  \param[in] b1 El valor que será escrito cuando se representa a un bit 1.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_set_bcd(PdsVector *Vector,PdsRaNatural n,PdsRaNatural b0,PdsRaNatural b1);


/** \fn int pds_vector_set_gray(PdsVector *Vector,PdsRaNatural n,PdsRaNatural b0,PdsRaNatural b1)
 *  \brief Escribe en formato Gray el numero n en el vector Vector.
 *  \param[in,out] Vector El vector a escribir.
 *  \param[in] n EL número decimal e escribir en Gray en el vector Vector
 *  \param[in] b0 El valor que será escrito cuando se representa a un bit 0.
 *  \param[in] b1 El valor que será escrito cuando se representa a un bit 1.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_set_gray(PdsVector *Vector,PdsRaNatural n,PdsRaNatural b0,PdsRaNatural b1);


/** \fn int pds_vector_cmp(const PdsVector *Vector1,const PdsVector *Vector2,PdsRaNatural *n)
 *  \brief Compara los vectores \b Vector1 y \b Vector2, retorna  en \b n la cantidad de elementos distintos.
 *  \param[in] Vector1 Un vector a comparar.
 *  \param[in] Vector2 Un vector a comparar.
 *  \param[out] n EL número de elementos diferentes entre los vectores.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector1==NULL o Vector2==NULL). 
 *  \ingroup PdsVectorGroup
 */
int pds_vector_cmp(const PdsVector *Vector1,const PdsVector *Vector2,PdsRaNatural *n);


/** \fn PdsVector *pds_vector_new_linspace(PdsRaReal x1,PdsRaReal x2,PdsRaNatural N
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con N elementos
 *  equidistantemente espaciados ((x2-x1)/(N-1)) entre x1 y x2 (inclusive).
 *  \param[in] N Es la longitud del vector.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_linspace(PdsRaReal x1,PdsRaReal x2,PdsRaNatural N);


/** \fn PdsVector *pds_vector_new_rectangular(PdsRaNatural N)
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con una ventana
 *  rectangular W.
 *  \f[ W[n]=1 ~~~ \forall 0 \leq n<N\f]
 *  \param[in] N Es la longitud del vector que representa la ventana de rectangular.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_rectangular(PdsRaNatural N);


/** \fn PdsVector *pds_vector_new_hamming(PdsRaNatural N)
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con una ventana
 *  de Hamming W.
 *  \f[ W[n]=0.54-0.46 cos(2 \frac{\pi n}{N-1}) ~~~ \forall 0 \leq n<N\f]
 *  \param[in] N Es la longitud del vector que representa la ventana de Hamming.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_hamming(PdsRaNatural N);


/** \fn PdsVector *pds_vector_new_blackman(PdsRaNatural N,PdsRaReal Alpha)
 *  \brief Crea un vector de tipo PdsVector e inicia sus valores con una ventana
 *  de Blackman.
 *  \f[ W[n]=\frac{1-Alpha}{2}-\frac{1}{2} cos(2 \frac{\pi n}{N-1}) +\frac{Alpha}{2} cos(4 \frac{\pi n}{N-1}) ~~~ \forall 0 \leq n<N\f]
 *  \param[in] N Es la longitud del vector que representa la ventana de Blackman.
 *  \return Un puntero al vector de tipo PdsVector.
 *  \ingroup PdsVectorGroup
 */
PdsVector *pds_vector_new_blackman(PdsRaNatural N,PdsRaReal Alpha);


/** \fn int pds_vector_octave_png_plot(const PdsVector *X,const PdsVector *Y,const char *labelx,const char *labely,const char *octfile,const char *pngfile)
 *  \brief Imprime un vector Y vs X en un archivo png usando como intermedio un 
 *  archivo *.m de octave.
 *  \param[in] X Eje X del vector a imprimir.
 *  Si X==NULL entonces se mostrara la gráfica usando el numero de muestras
 *  como eje X.
 *  \param[in] Y Eje Y del vector a imprimir.
 *  \param[in] labelx Etiqueta del eje X.
 *  \param[in] labely Etiqueta del eje Y.
 *  \param[in] octfile Nombre del archivo *.m de octave que se usará de intermediario.
 *  Si ostfile==NULL entonces se creará un fichero con nombre aleatorio y se 
 *  borrará al terminar de crear el archivo png.
 *  \param[in] pngfile Nombre del archivo *.png de salida.
 *  \return TRUE si todo fue bien o FALSE si no, ejemplo punteros igual a NULL.
 *  \ingroup PdsVectorGroup
 */
int pds_vector_octave_png_plot(const PdsVector *X,const PdsVector *Y,const char *labelx,const char *labely,const char *octfile,const char *pngfile);


/** \fn void pds_vector_free(PdsVector *Vector)
 *  \brief Libera la memoria de un vector de tipo puntero PdsVector.
 *  \param[in,out] Vector El vector a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsVectorGroup
 */
void pds_vector_free(PdsVector *Vector);


/** \fn void pds_vector_destroy(PdsVector **Vector)
 *  \brief Teniendo un vector \b Vector de tipo <code>PdsVector *Vector</code>, entonces
\code{.c}
pds_vector_destroy(&Vector);
\endcode
 *  Libera la memoria usada por \b Vector y apunta \b Vector a NULL.
 *  Apuntar Vector a NULL es una medida de seguridad para evitar
 *  usar el vector después de liberado, así este no apuntará a un lugar que no le
 *  corresponde mas.
 *  \param[in,out] Vector Un puntero al vector a liberar y limpiar.
 *  \return No retorna valor.
 *  \ingroup PdsVectorGroup
 */
void pds_vector_destroy(PdsVector **Vector);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif


